@@ -1,6 +1,5 @@
-# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
-# If a copy of the MPL was not distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/.
-# © 2025 cswimr
+# SPDX-FileCopyrightText: 2025 cswimr <copyright@csw.im>
+# SPDX-License-Identifier: MPL-2.0
 
 from importlib.util import find_spec
 from warnings import warn
@@ -16,7 +15,7 @@ if not find_spec("class_registry"):
 from .cog import SentinelCog
 from .db import AbstractPartial, Change, Moderation, PartialChannel, PartialGuild, PartialUser
 from .exceptions import HandlerError, LoggedHandlerError, UpsertError
-from .type import ModerationType, moderation_type_registry
+from .type import ModerationMetadataEntry, ModerationType, moderation_type_registry
 
 __all__ = [
     "AbstractPartial",
@@ -26,6 +25,7 @@ __all__ = [
     "PartialChannel",
     "PartialGuild",
     "PartialUser",
+    "ModerationMetadataEntry",
     "ModerationType",
     "moderation_type_registry",
     "HandlerError",
