@@ -1,6 +1,5 @@
-# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
-# If a copy of the MPL was not distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/.
-# © 2025 cswimr
+# SPDX-FileCopyrightText: 2025 cswimr <copyright@csw.im>
+# SPDX-License-Identifier: MPL-2.0
 
 """Cog class for cogs using the Sentinel moderation system."""
 
@@ -101,7 +100,7 @@ class SentinelCog(Cog):
                 cog_instance=self._db_path, tables=[Change, Moderation, PartialChannel, PartialGuild, PartialUser], trace=True
             )
 
-        for moderation_type in self.moderation_types:
+        for moderation_type in self.sentinel_moderation_types:
             if moderation_type.key in self.moderation_type_registry:
                 conflict = self.moderation_type_registry[moderation_type.key]
                 cls = conflict.__class__
@@ -119,14 +118,14 @@ class SentinelCog(Cog):
             self.moderation_type_registry.register(key=moderation_type.key)(moderation_type)
             self._sentinel_logger.verbose("Registered moderation type with key '%s'", moderation_type.key)
 
-        for old, new in self.moderation_type_migrations.items():
+        for old, new in self.sentinel_moderation_type_migrations.items():
             await Moderation.update({Moderation.type_key: new.key}).where(Moderation.type_key == old)
 
             guilds = [guild async for guild in self.bot.fetch_guilds() if not await self.bot.cog_disabled_in_guild(self, guild)]
 
             for guild in guilds:
-                old_config = self.get_type_config(guild, old)
-                new_config = self.get_type_config(guild, new)
+                old_config = self.get_sentinel_type_config(guild, old)
+                new_config = self.get_sentinel_type_config(guild, new)
                 await new_config.set(await old_config.all())
                 await old_config.clear()
 
@@ -141,7 +140,7 @@ class SentinelCog(Cog):
             as this method cleans up the Sentinel expiry loop and moderation type registry, and must be ran.
         """
         await super().cog_unload()
-        for moderation_type in self.moderation_types:
+        for moderation_type in self.sentinel_moderation_types:
             if moderation_type.key not in self.moderation_type_registry:
                 self._sentinel_logger.warning(
                     "Moderation type with key '%s' does not exist in the type registry, skipping unregistration.", moderation_type.key
@@ -179,6 +178,7 @@ class SentinelCog(Cog):
 
     def _sentinel_register_config(self) -> None:
         self.sentinel_config.register_guild(
+            default_reason="No reason provided.",  # str
             show_moderator=True,  # boolean
             use_discord_permissions=True,  # boolean
             respect_hierarchy=True,  # boolean
@@ -192,6 +192,7 @@ class SentinelCog(Cog):
         )
 
         moderation_type = {
+            "default_reason": None,  # str | None
             "show_in_history": True,  # boolean
             "show_moderator": None,  # boolean | None
             "use_discord_permissions": None,  # boolean | None
@@ -205,7 +206,7 @@ class SentinelCog(Cog):
         self.sentinel_config.register_custom(group_identifier="types", **moderation_type)
 
     @property
-    def moderation_types(self) -> list[type[ModerationType]]:
+    def sentinel_moderation_types(self) -> list[type[ModerationType]]:
         """Return a set of moderation types that are added by this cog.
 
         If you are adding any custom moderation types, you **must** override this method to return your new moderation types.
@@ -225,7 +226,7 @@ class SentinelCog(Cog):
         return []
 
     @property
-    def moderation_type_migrations(self) -> Mapping[str, type[ModerationType]]:
+    def sentinel_moderation_type_migrations(self) -> Mapping[str, type[ModerationType]]:
         """Return a mapping of moderation type keys that should be replaced within the database to new ModerationTypes.
 
         Migrations will occur on cog load. Please take care to not migrate the types of other cogs utilizing Sentinel unless you expressly want to.
@@ -259,7 +260,7 @@ class SentinelCog(Cog):
         """
         return {name: cog for name, cog in self.bot.cogs.items() if isinstance(cog, SentinelCog)}
 
-    def get_type_config(self, guild: Guild | int, moderation_type: ModerationType | type[ModerationType] | str) -> Group:
+    def get_sentinel_type_config(self, guild: Guild | int, moderation_type: ModerationType | type[ModerationType] | str) -> Group:
         """Retrieve a [`Group`][redbot.core.config.Group] that contains configuration for a specific moderation type.
 
         Args:
@@ -281,7 +282,7 @@ class SentinelCog(Cog):
 
         return self.sentinel_config.custom("types", str(guild_id), key)
 
-    async def get_type_config_with_fallback(self, guild: Guild | int, moderation_type: ModerationType, attr: str) -> Any:
+    async def get_sentinel_type_config_with_fallback(self, guild: Guild | int, moderation_type: ModerationType, attr: str) -> Any:
         """Retrieve a configuration value for a specific moderation type, falling back to the guild-level config if the type-level config is missing.
 
         Args:
@@ -300,7 +301,7 @@ class SentinelCog(Cog):
         else:
             guild_id = guild
 
-        custom = self.get_type_config(guild, moderation_type)
+        custom = self.get_sentinel_type_config(guild, moderation_type)
         result: Any = None
         try:
             func = getattr(custom, attr)
@@ -322,7 +323,7 @@ class SentinelCog(Cog):
 
         return result
 
-    async def log_channel(self, guild: Guild) -> abc.GuildChannel | Thread | None:
+    async def sentinel_log_channel(self, guild: Guild) -> abc.GuildChannel | Thread | None:
         """Retrieve the moderation logging channel for a guild.
 
         Args:
@@ -342,12 +343,13 @@ class SentinelCog(Cog):
         except NotFound:
             return None
 
-    async def moderate(
+    async def sentinel_moderate(
         self,
         ctx: commands.Context | Interaction,
         target: Member | User | abc.GuildChannel | Thread,
         moderation_type: ModerationType | str,
         silent: bool = MISSING,
+        reason: str | None = None,
         **kwargs: Any,
     ) -> None | Moderation:
         """Moderate a user.
@@ -362,6 +364,7 @@ class SentinelCog(Cog):
             target: The target user or channel to moderate.
             moderation_type: The moderation type (handler) to use.
             silent: Whether or not to message the target.
+            reason: The reason to give this moderation. Defaults to guild configuration when not provided.
             **kwargs: Any additional keyword arguments to pass to the handler method.
 
         Returns:
@@ -381,7 +384,7 @@ class SentinelCog(Cog):
             return None
 
         try:
-            await self.check_moddable(target=target, ctx=ctx, moderation_type=moderation_type)
+            await self.sentinel_check_moddable(target=target, ctx=ctx, moderation_type=moderation_type)
         except (PermissionError, ValueError) as err:
             await send_error(ctx, content=str(err), ephemeral=True)
             return None
@@ -393,13 +396,16 @@ class SentinelCog(Cog):
             return None
 
         if silent == MISSING:
-            silent = await self.get_type_config_with_fallback(ctx.guild, moderation_type, "dm_users")
+            silent = await self.get_sentinel_type_config_with_fallback(ctx.guild, moderation_type, "dm_users")
+
+        if not reason:
+            reason = await self.get_sentinel_type_config_with_fallback(ctx.guild, moderation_type, "default_reason")
 
         try:
             if not isinstance(target, (abc.GuildChannel, Thread)):
                 kwargs["silent"] = silent if moderation_type.removes_from_guild else True
 
-            moderation = await moderation_handler(cog=self, ctx=ctx, target=target, **kwargs)
+            moderation = await moderation_handler(cog=self, ctx=ctx, target=target, reason=reason, **kwargs)
         except HandlerError as err:
             await err.send(ctx)
             return None
@@ -414,20 +420,20 @@ class SentinelCog(Cog):
 
         if not moderation_type.removes_from_guild and not silent:
             try:
-                await self.contact_target(moderation)
+                await self.sentinel_contact_target(moderation)
             except (ConfigurationError, TypeError) as err:
                 self._sentinel_logger.warning("Unable to contact target '%s'!", target.id, exc_info=err)
             except Forbidden:
                 pass
 
         try:
-            await self.send_log(moderation)
+            await self.sentinel_send_log(moderation)
         except ConfigurationError:
             pass
 
         return moderation
 
-    def check_permissions(
+    def sentinel_check_permissions(
         self,
         user: Member | abc.User | PartialUser,
         required_permissions: Permissions,
@@ -500,7 +506,7 @@ class SentinelCog(Cog):
             msg = f"{who} do not have the {perms_list} {part} required for this action."
             raise PermissionError(msg)
 
-    async def check_moddable(
+    async def sentinel_check_moddable(
         self,
         ctx: commands.Context,
         target: User | Member | abc.GuildChannel | Thread,
@@ -543,13 +549,13 @@ class SentinelCog(Cog):
 
         is_channel = isinstance(target, (abc.GuildChannel, Thread))
 
-        self.check_permissions(
+        self.sentinel_check_permissions(
             user=me, required_permissions=moderation_type.permissions, guild=ctx.guild, channel=target if is_channel else ctx.channel
         )
 
-        use_discord_permissions = await self.get_type_config_with_fallback(ctx.guild, moderation_type, "use_discord_permissions")
+        use_discord_permissions = await self.get_sentinel_type_config_with_fallback(ctx.guild, moderation_type, "use_discord_permissions")
         if use_discord_permissions is True:
-            self.check_permissions(
+            self.sentinel_check_permissions(
                 user=ctx.author, required_permissions=moderation_type.permissions, guild=ctx.guild, channel=target if is_channel else ctx.channel
             )
 
@@ -587,7 +593,7 @@ class SentinelCog(Cog):
                     msg = f"You cannot moderate members with the {formatted} role{plural}, because {part} immune role{plural}!"
                     raise PermissionError(msg)
 
-    async def contact_target(
+    async def sentinel_contact_target(
         self,
         moderation: Moderation,
         *,
@@ -663,18 +669,18 @@ class SentinelCog(Cog):
 
             kwargs["file"] = get_asset_as_file(filename="arrow.png", description="Arrow")
 
-        show_moderator = await self.get_type_config_with_fallback(guild, moderation.type, "show_moderator")
+        show_moderator = await self.get_sentinel_type_config_with_fallback(guild, moderation.type, "show_moderator")
         if show_moderator and moderator is not None:
             embed.add_field(name="Moderator", value=f"`{moderator.name} ({moderator.user_id})`", inline=False)
 
         if moderation.reason:
             embed.add_field(name="Reason", value=f"`{moderation.reason}`", inline=False)
 
-        if support_message := await self.get_type_config_with_fallback(guild, moderation.type, "support_message"):
+        if support_message := await self.get_sentinel_type_config_with_fallback(guild, moderation.type, "support_message"):
             embed.add_field(name="Support", value=support_message, inline=False)
 
-        button_label = await self.get_type_config_with_fallback(guild, moderation.type, "button_label")
-        button_url = await self.get_type_config_with_fallback(guild, moderation.type, "button_url")
+        button_label = await self.get_sentinel_type_config_with_fallback(guild, moderation.type, "button_label")
+        button_url = await self.get_sentinel_type_config_with_fallback(guild, moderation.type, "button_url")
         if button_label and button_url:
             view = ui.View()
             view.add_item(ui.Button(label=button_label, url=button_url))
@@ -682,7 +688,7 @@ class SentinelCog(Cog):
 
         return await target.send(embed=embed, **kwargs)
 
-    async def send_log(self, moderation: Moderation) -> Message:
+    async def sentinel_send_log(self, moderation: Moderation) -> Message:
         """Send a moderation log to a guild's configured moderation channel.
 
         Args:
@@ -695,10 +701,10 @@ class SentinelCog(Cog):
             The resulting logging message.
         """
         guild = await (await moderation.guild()).fetch(self.bot)
-        if not (log_channel := await self.log_channel(guild)) or not isinstance(log_channel, abc.Messageable):
+        if not (log_channel := await self.sentinel_log_channel(guild)) or not isinstance(log_channel, abc.Messageable):
             msg = "Logging is disabled!"
             raise ConfigurationError(msg)
-        embed = await self.case_embed(moderation, color=await self.bot.get_embed_color(location=log_channel))
+        embed = await self.sentinel_case_embed(moderation, color=await self.bot.get_embed_color(location=log_channel))
         return await log_channel.send(embed=embed)
 
     @staticmethod
@@ -722,7 +728,7 @@ class SentinelCog(Cog):
         msg = f"{partial_object!r} is not a supported type! Got: {type(partial_object)!r}"
         raise TypeError(msg)
 
-    async def case_embed(self, moderation: Moderation, color: Color | None = None) -> Embed:
+    async def sentinel_case_embed(self, moderation: Moderation, color: Color | None = None) -> Embed:
         """Create a case embed from a Moderation object.
 
         Args:
@@ -779,18 +785,17 @@ class SentinelCog(Cog):
             )
             embed.description += f"\n{cf.bold('Duration:')} {duration_embed}\n{cf.bold('Expired:')} {moderation.expired}"
 
-        if moderation.meta:
-            for key, value in moderation.meta.items():
-                if key == "imported_timestamp":
-                    timestamp = datetime.fromisoformat(value)
-                    embed.description += f"\n{cf.bold(f'{title(key)}:')} {cf.format_datetime(dt=timestamp)}"
-                else:
-                    try:
-                        if func := moderation.type.history_metadata.get(key, None):
-                            value = func(value)  # noqa: PLW2901
-                    except RegistryKeyError:
-                        pass
-                    embed.description += f"\n{cf.bold(f'{title(key)}:')} {value}"
+        if value := moderation.meta.get("imported_timestamp"):
+            timestamp = datetime.fromisoformat(value)
+            embed.description += f"\n{cf.bold('Imported Timestamp:')} {cf.format_datetime(dt=timestamp)}"
+
+        try:
+            moderation_type = moderation.type
+            for metadata in moderation_type.metadata:
+                if value := metadata.fetch_from_moderation(moderation):
+                    embed.description += f"\n{cf.bold(f'{title(metadata.human_name)}:')} {value}"
+        except RegistryKeyError:
+            pass
 
         if moderation.reason:
             embed.add_field(name="Reason", value=cf.box(truncate_string(moderation.reason, 1010)), inline=False)
@@ -809,7 +814,7 @@ class SentinelCog(Cog):
                 pass
         return embed
 
-    async def changes_menu(self, moderation: Moderation, color: Color | None = None) -> SimpleMenu:
+    async def sentinel_changes_menu(self, moderation: Moderation, color: Color | None = None) -> SimpleMenu:
         """Create a menu containing all of the historical changes to a Moderation.
 
         Args:
@@ -869,7 +874,7 @@ class SentinelCog(Cog):
 
         return SimpleMenu(pages)
 
-    async def history(
+    async def sentinel_history_menu(
         self,
         ctx: commands.Context,
         *,
@@ -969,7 +974,7 @@ class SentinelCog(Cog):
         elif types is not True:
             raw_types = []
             for moderation_type in self.moderation_type_registry.classes():
-                config = self.get_type_config(ctx.guild, moderation_type)
+                config = self.get_sentinel_type_config(ctx.guild, moderation_type)
                 if await config.show_in_history() is True:
                     raw_types.append(moderation_type.key)
             where.append(Moderation.type_key.is_in(raw_types))
@@ -1075,10 +1080,9 @@ class SentinelCog(Cog):
                     moderation_type = None
 
                 if moderation_type:
-                    for key, content in moderation.meta.items():
-                        if key in moderation.type.history_metadata.keys():
-                            func = moderation.type.history_metadata[key]
-                            value += f"{cf.bold(title(key) + ':')} {func(content)}\n"
+                    for meta in [meta for meta in moderation_type.metadata if meta.show_in_history]:
+                        if meta_value := meta.fetch_from_moderation(moderation):
+                            value += f"{cf.bold(title(meta.human_name) + ':')} {meta_value}\n"
 
                 embed.add_field(name=f"Case {moderation.id:,} ({title(moderation.type_key)})", value=value, inline=inline)
             pages.append(embed)
