@@ -1,14 +1,12 @@
-# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
-# If a copy of the MPL was not distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/.
-# © 2025 cswimr
-
+# SPDX-FileCopyrightText: 2025 cswimr <copyright@csw.im>
+# SPDX-License-Identifier: MPL-2.0
 """Sentinel database table models."""
 
 from abc import abstractmethod
 from datetime import UTC, datetime, timedelta
 from decimal import Decimal
 from enum import StrEnum
-from typing import TYPE_CHECKING, Any, Literal, Self, overload
+from typing import TYPE_CHECKING, Any, Literal, Never, Self, overload
 
 import orjson
 import rich.repr
@@ -21,27 +19,13 @@ from redbot.core.bot import Red
 from typing_extensions import override
 
 from tidegear.exceptions import NotFoundError
-from tidegear.sentinel.exceptions import NotReadyError, UpsertError
+from tidegear.sentinel.exceptions import NotReadyError, UnsetError, UpsertError
 from tidegear.sentinel.type import ModerationType, moderation_type_registry
 
 if TYPE_CHECKING:
     from tidegear.sentinel.cog import SentinelCog
 
 
-def _clean_foreign_keys(obj: dict) -> dict:
-    cleaned = {}
-    for k, v in obj.items():
-        if isinstance(v, dict):
-            nested = _clean_foreign_keys(v)
-            if all(value is None for value in nested.values()):
-                cleaned[k] = None
-            else:
-                cleaned[k] = nested
-        else:
-            cleaned[k] = v
-    return cleaned
-
-
 class Table(BaseTable):
     """Subclass of Piccolo's Table class that allows for easier pretty printing of table rows."""
 
@@ -59,6 +43,18 @@ class Table(BaseTable):
 
     __rich_repr__.angular = True  # pyright: ignore[reportFunctionMemberAccess]
 
+    def is_null(self) -> bool:
+        """Check if the current row is null, or empty.
+        Helps with weirdness related to the Piccolo [`all_related()`][piccolo.table.Table.all_related] function.
+
+        Returns:
+            Whether or not all entries within the current row are null.
+        """
+        for value in self.to_dict().values():
+            if value is not None:
+                return False
+        return True
+
     @overload
     def json(self, *, string: Literal[False] = False, indent: bool = ..., options: int = ...) -> bytes: ...
 
@@ -91,15 +87,29 @@ class Table(BaseTable):
             return dump.decode("utf-8")
         return dump
 
-    @staticmethod
-    def _json_serializer(obj: object) -> object:
+    @classmethod
+    def _json_serializer(cls, obj: object) -> object:
         if isinstance(obj, Table):
-            return _clean_foreign_keys(obj.to_dict())
+            return cls._clean_foreign_keys(obj.to_dict())
         if isinstance(obj, Decimal):
             return float(obj)
         msg = f"Type {type(obj)} is not serializable!"
         raise TypeError(msg)
 
+    @classmethod
+    def _clean_foreign_keys(cls, obj: dict) -> dict:
+        cleaned = {}
+        for k, v in obj.items():
+            if isinstance(v, dict):
+                nested = cls._clean_foreign_keys(v)
+                if all(value is None for value in nested.values()):
+                    cleaned[k] = None
+                else:
+                    cleaned[k] = nested
+            else:
+                cleaned[k] = v
+        return cleaned
+
 
 class AbstractPartial:
     """An abstract class for Partials, detailing methods that should always be implemented within a Partial.
@@ -418,7 +428,9 @@ class PartialChannel(Table, AbstractPartial):
         if cache and "_guild_obj" in self.__dict__:
             return self._guild_obj
 
-        if not (guild := await PartialGuild.objects().where(PartialGuild.id == self.guild_id).first()):
+        if isinstance(self.guild_id, PartialGuild):  # support both initialized PartialGuild objects and raw ints
+            guild = self.guild_id
+        elif not (guild := await PartialGuild.objects(PartialGuild.all_related()).where(PartialGuild.id == self.guild_id).first()):
             msg = f"No guild exists in the database with id {self.guild_id}!"
             raise NotFoundError(msg)
 
@@ -507,7 +519,9 @@ class Change(Table):
         if cache and "_moderation_obj" in self.__dict__:
             return self._moderation_obj
 
-        if not (mod := await Moderation.objects().where(Moderation.id == self.moderation_id).first()):
+        if isinstance(self.moderation_id, Moderation):  # support both initialized Moderation objects and raw ints
+            mod = self.moderation_id
+        elif not (mod := await Moderation.objects().where(Moderation.id == self.moderation_id).first()):
             msg = f"Moderation with id {self.moderation_id} does not exist in the database!"
             raise NotFoundError(msg)
 
@@ -524,17 +538,32 @@ class Change(Table):
         Raises:
             NotFoundError: If the [`PartialUser`][tidegear.sentinel.PartialUser] tied to this change no longer exists in the database.
                 This should be reported as a bug if it occurs, as there's a foreign key constraint that should prevent this on the table itself.
+            UnsetError: If the change has no moderator set.
+
 
         Returns:
             The user who made this change.
-        """
+        """  # noqa: DOC502
         if cache and "_moderator_obj" in self.__dict__:
             return self._moderator_obj
 
-        if not (user := await PartialUser.objects().where(PartialUser.id == self.moderator_id).first()):
-            msg = f"PartialUser with id {self.moderator_id} does not exist in the database!"
+        def unset() -> Never:
+            msg = f"Change {self.id:,} does not have a moderator set"
+            raise UnsetError(msg)
+
+        def not_found() -> Never:
+            msg = f"Could not find a PartialUser in the database with id {self.moderator_id}"
             raise NotFoundError(msg)
 
+        if self.moderator_id is None:
+            unset()
+        elif isinstance(self.moderator_id, PartialUser):  # support both initialized PartialUser objects and raw ints
+            if self.moderator_id.is_null():
+                unset()
+            user = self.moderator_id
+        elif not (user := await PartialUser.objects(PartialUser.all_related()).where(PartialUser.id == self.moderator_id).first()):
+            not_found()
+
         self._moderator_obj = user
         return user
 
@@ -653,17 +682,31 @@ class Moderation(Table):
         Raises:
             NotFoundError: If the [`PartialGuild`][tidegear.sentinel.PartialGuild] tied to this moderation no longer exists in the database.
                 This should be reported as a bug if it occurs, as there's a foreign key constraint that should prevent this on the table itself.
+            UnsetError: If the moderation has no guild set. This indicates a bug in the moderation handler for whatever moderation type this is.
 
         Returns:
             The PartialGuild that this moderation belongs to.
-        """
+        """  # noqa: DOC502
         if cache and "_guild_obj" in self.__dict__:
             return self._guild_obj
 
-        if not (guild := await PartialGuild.objects().where(PartialGuild.id == self.guild_id).first()):
+        def unset() -> Never:
+            msg = f"Moderation {self.id:,} does not have a guild set"
+            raise UnsetError(msg)
+
+        def not_found() -> Never:
             msg = f"Could not find a PartialGuild in the database with id {self.guild_id}"
             raise NotFoundError(msg)
 
+        if self.guild_id is None:
+            unset()
+        elif isinstance(self.guild_id, PartialGuild):  # support both initialized PartialGuild objects and raw ints
+            if self.guild_id.is_null():
+                unset()
+            guild = self.guild_id
+        elif not (guild := await PartialGuild.objects(PartialGuild.all_related()).where(PartialGuild.id == self.guild_id).first()):
+            not_found()
+
         self._guild_obj = guild
         return guild
 
@@ -677,17 +720,31 @@ class Moderation(Table):
         Raises:
             NotFoundError: If the [`PartialUser`][tidegear.sentinel.PartialUser] tied to this moderation no longer exists in the database.
                 This should be reported as a bug if it occurs, as there's a foreign key constraint that should prevent this on the table itself.
+            UnsetError: If the moderation has no moderator set. This indicates a bug in the moderation handler for whatever moderation type this is.
 
         Returns:
             The PartialUser who is credited with this moderation.
-        """
+        """  # noqa: DOC502
         if cache and "_moderator_obj" in self.__dict__:
             return self._moderator_obj
 
-        if not (user := await PartialUser.objects().where(PartialUser.id == self.moderator_id).first()):
+        def unset() -> Never:
+            msg = f"Moderation {self.id:,} does not have a moderator set"
+            raise UnsetError(msg)
+
+        def not_found() -> Never:
             msg = f"Could not find a PartialUser in the database with id {self.moderator_id}"
             raise NotFoundError(msg)
 
+        if self.moderator_id is None:
+            unset()
+        elif isinstance(self.moderator_id, PartialUser):  # support both initialized PartialUser objects and raw ints
+            if self.moderator_id.is_null():
+                unset()
+            user = self.moderator_id
+        elif not (user := await PartialUser.objects(PartialUser.all_related()).where(PartialUser.id == self.moderator_id).first()):
+            not_found()
+
         self._moderator_obj = user
         return user
 
@@ -702,7 +759,7 @@ class Moderation(Table):
             NotFoundError: If the [`PartialUser`][tidegear.sentinel.PartialUser] or [`PartialChannel`][tidegear.sentinel.PartialChannel]
                 tied to this moderation no longer exists in the database.
                 This should be reported as a bug if it occurs, as there's a foreign key constraint that should prevent this on the table itself.
-            ValueError: If neither `target_user` or `target_channel` are set on the moderation object.
+            UnsetError: If neither `target_user` or `target_channel` are set on the moderation object.
 
         Returns:
             The PartialUser or PartialChannel that was targeted by this moderation.
@@ -710,19 +767,29 @@ class Moderation(Table):
         if cache and "_target_obj" in self.__dict__:
             return self._target_obj
 
+        result: PartialChannel | PartialUser | None = None
+
         if self.target_user_id is not None:
-            if not (result := await PartialUser.objects().where(PartialUser.id == self.target_user_id).first()):
+            if isinstance(self.target_user_id, PartialUser):  # support both initialized PartialUser objects and raw ints
+                if not self.target_user_id.is_null():
+                    result = self.target_user_id
+            elif not (result := await PartialUser.objects(PartialUser.all_related()).where(PartialUser.id == self.target_user_id).first()):
                 msg = f"Could not find a PartialUser in the database with id {self.target_user_id}"
                 raise NotFoundError(msg)
 
-        elif self.target_channel_id is not None:
-            if not (result := await PartialChannel.objects().where(PartialChannel.id == self.target_channel_id).first()):
+        if not result and self.target_channel_id is not None:
+            if isinstance(self.target_channel_id, PartialChannel):  # support both initialized PartialChannel objects and raw ints
+                if not self.target_channel_id.is_null():
+                    result = self.target_channel_id
+            elif not (
+                result := await PartialChannel.objects(PartialChannel.all_related()).where(PartialChannel.id == self.target_channel_id).first()
+            ):
                 msg = f"Could not find a PartialChannel in the database with id {self.target_channel_id}"
                 raise NotFoundError(msg)
 
-        else:
+        if not result:
             msg = "Neither target_user nor target_channel are set!"
-            raise ValueError(msg)
+            raise UnsetError(msg)
 
         self._target_obj = result
         return result
@@ -737,17 +804,31 @@ class Moderation(Table):
         Raises:
             NotFoundError: If the [`PartialUser`][tidegear.sentinel.PartialUser] tied to this moderation no longer exists in the database.
                 This should be reported as a bug if it occurs, as there's a foreign key constraint that should prevent this on the table itself.
+            UnsetError: If the moderation has no resolver set.
 
         Returns:
             The PartialUser who is credited with resolving this moderation.
-        """
+        """  # noqa: DOC502
         if cache and "_resolver_obj" in self.__dict__:
             return self._resolver_obj
 
-        if not (user := await PartialUser.objects().where(PartialUser.id == self.resolver_id).first()):
+        def unset() -> Never:
+            msg = f"Moderation {self.id:,} does not have a resolver set"
+            raise UnsetError(msg)
+
+        def not_found() -> Never:
             msg = f"Could not find a PartialUser in the database with id {self.resolver_id}"
             raise NotFoundError(msg)
 
+        if self.resolver_id is None:
+            unset()
+        elif isinstance(self.resolver_id, PartialUser):  # support both initialized PartialUser objects and raw ints
+            if self.resolver_id.is_null():
+                unset()
+            user = self.resolver_id
+        elif not (user := await PartialUser.objects(PartialUser.all_related()).where(PartialUser.id == self.resolver_id).first()):
+            not_found()
+
         self._resolver_obj = user
         return user
 
